#ifndef _ICONV_H_
#define _ICONV_H_

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>

typedef void *iconv_t;

typedef size_t
           (*iconv_func)(iconv_t, const char **, size_t *, char **, size_t *);

iconv_t iconv_open(const char *to, const char *from);
int iconv_close(iconv_t cd);
int iconv_byteratio(iconv_t cd);
const char *iconv_error(int errnum);

#define iconv(cd, inbuf, inbytesleft, outbuf, outbytesleft) \
            ((**(iconv_func *)(cd))((cd), (inbuf), (inbytesleft), (outbuf), \
                                    (outbytesleft)))

typedef ssize_t (*iconv_stream_func)(void *d, void *buf, size_t nbytes);

typedef struct {
    iconv_t cd;
    size_t chars;
    size_t in_bytes;
    size_t out_bytes;
    char *buffer;
    char *buf_ptr;
    void *handle;
    iconv_stream_func method;
} iconv_stream;

iconv_stream *iconv_stream_open(iconv_t cd, void *handle,
                                iconv_stream_func method);
void iconv_stream_close(iconv_stream *stream);

#define iconv_ostream_open(cd, handle) \
            iconv_stream_open((cd), (void *)(handle), \
                              (iconv_stream_func)write)
iconv_stream *iconv_ostream_fopen(iconv_t cd, FILE *handle);

ssize_t iconv_write(void *stream, const void *buf, size_t nbytes);
ssize_t iconv_bwrite(void *stream, const void *buf, size_t nbytes);

#endif
